@@ -1,9 +1,15 @@
 from __future__ import print_function
 from __future__ import with_statement
 
+import dataclasses
+import multiprocessing
+import time
+import typing
+
 import pytest
 import portalocker
 from portalocker import utils
+from portalocker import LockFlags
 
 
 def test_exceptions(tmpfile):
@@ -36,8 +42,10 @@ def test_with_timeout(tmpfile):
     with pytest.raises(portalocker.AlreadyLocked):
         with portalocker.Lock(tmpfile, timeout=0.1) as fh:
             print('writing some stuff to my cache...', file=fh)
-            with portalocker.Lock(tmpfile, timeout=0.1, mode='wb',
-                                  fail_when_locked=True):
+            with portalocker.Lock(
+                tmpfile, timeout=0.1, mode='wb',
+                fail_when_locked=True
+            ):
                 pass
             print('writing more stuff to my cache...', file=fh)
 
@@ -206,3 +214,111 @@ def test_shared(tmpfile):
     portalocker.unlock(f)
     f.close()
 
+
+def test_blocking_timeout(tmpfile):
+    flags = LockFlags.SHARED
+
+    with pytest.warns(UserWarning):
+        with portalocker.Lock(tmpfile, timeout=5, flags=flags):
+            pass
+
+    lock = portalocker.Lock(tmpfile, flags=flags)
+    with pytest.warns(UserWarning):
+        lock.acquire(timeout=5)
+
+
+def shared_lock(filename, **kwargs):
+    with portalocker.Lock(
+        filename,
+        timeout=0.1,
+        fail_when_locked=False,
+        flags=LockFlags.SHARED | LockFlags.NON_BLOCKING,
+    ):
+        time.sleep(0.2)
+        return True
+
+
+def shared_lock_fail(filename, **kwargs):
+    with portalocker.Lock(
+        filename,
+        timeout=0.1,
+        fail_when_locked=True,
+        flags=LockFlags.SHARED | LockFlags.NON_BLOCKING,
+    ):
+        time.sleep(0.2)
+        return True
+
+
+def exclusive_lock(filename, **kwargs):
+    with portalocker.Lock(
+        filename,
+        timeout=0.1,
+        fail_when_locked=False,
+        flags=LockFlags.EXCLUSIVE | LockFlags.NON_BLOCKING,
+    ):
+        time.sleep(0.2)
+        return True
+
+
+@dataclasses.dataclass(order=True)
+class LockResult:
+    exception_class: typing.Union[typing.Type[Exception], None] = None
+    exception_message: typing.Union[str, None] = None
+    exception_repr: typing.Union[str, None] = None
+
+
+def lock(
+    filename: str,
+    fail_when_locked: bool,
+    flags: LockFlags
+) -> LockResult:
+    # Returns a case of True, False or FileNotFound
+    # https://thedailywtf.com/articles/what_is_truth_0x3f_
+    # But seriously, the exception properties cannot be safely pickled so we
+    # only return string representations of the exception properties
+    try:
+        with portalocker.Lock(
+            filename,
+            timeout=0.1,
+            fail_when_locked=fail_when_locked,
+            flags=flags,
+        ):
+            time.sleep(0.2)
+            return LockResult()
+
+    except Exception as exception:
+        # The exceptions cannot be pickled so we cannot return them through
+        # multiprocessing
+        return LockResult(
+            type(exception),
+            str(exception),
+            repr(exception),
+        )
+
+
+@pytest.mark.parametrize('fail_when_locked', [True, False])
+def test_shared_processes(tmpfile, fail_when_locked):
+    flags = LockFlags.SHARED | LockFlags.NON_BLOCKING
+
+    with multiprocessing.Pool(processes=2) as pool:
+        args = tmpfile, fail_when_locked, flags
+        results = pool.starmap_async(lock, 2 * [args])
+
+        for result in results.get(timeout=3):
+            assert result == LockResult()
+
+
+@pytest.mark.parametrize('fail_when_locked', [True, False])
+def test_exclusive_processes(tmpfile, fail_when_locked):
+    flags = LockFlags.EXCLUSIVE | LockFlags.NON_BLOCKING
+
+    with multiprocessing.Pool(processes=2) as pool:
+        # filename, fail_when_locked, flags
+        args = tmpfile, fail_when_locked, flags
+        a, b = pool.starmap_async(lock, 2 * [args]).get(timeout=3)
+
+        assert not a.exception_class or not b.exception_class
+        assert issubclass(
+            a.exception_class or b.exception_class,
+            portalocker.LockException
+        )
